#ifndef XMLVISITORS_H
#define XMLVISITORS_H

#include "File.h"
#include "XMLElements.h"

//----------------------------------------------------------------------------
/**
 * Base class for all XML visitors.
 */
class XMLVisitor
{
  public:
    //------------------------------------------------------------------------
    virtual ~XMLVisitor() {}

    //------------------------------------------------------------------------
    inline void visit(XMLProperty *a) { do_visit(a); }
    inline void visit(XMLNode *n) { do_visit(n); }

  protected:
    //------------------------------------------------------------------------
    XMLVisitor() {}

  private:
    //------------------------------------------------------------------------
    virtual void do_visit(XMLProperty *a) = 0;
    virtual void do_visit(XMLNode *n) = 0;
};


//----------------------------------------------------------------------------
/**
 * Base class for all XML const visitors.
 */
class XMLConstVisitor
{
  public:
    //------------------------------------------------------------------------
    virtual ~XMLConstVisitor() {}

    //------------------------------------------------------------------------
    inline void visit(const XMLProperty *a) { do_visit(a); }
    inline void visit(const XMLNode *n) { do_visit(n); }

  protected:
    //------------------------------------------------------------------------
    XMLConstVisitor() {}

  private:
    //------------------------------------------------------------------------
    virtual void do_visit(const XMLProperty *a) = 0;
    virtual void do_visit(const XMLNode *n) = 0;
};


//----------------------------------------------------------------------------
/**
 * This visitor is used to write the content of an XMLNode to a file.
 */
class XMLWriteToFileVisitor : public XMLConstVisitor
{
  public:
    //------------------------------------------------------------------------
    /**
     * @param name The name of the file to write to.
     * @throw SyscallException if the file cannot be opened for writing.
     */
    XMLWriteToFileVisitor(const char *name) throw (SyscallException);
    ~XMLWriteToFileVisitor();

  private:
    //------------------------------------------------------------------------
    void do_visit(const XMLProperty *a);
    void do_visit(const XMLNode *n);

    void writeIndentation();

    //------------------------------------------------------------------------
    File m_file;
    int m_level;
};

#endif //XMLVISITORS_H
