#ifndef PARTICLEFOUNTAINS_H
#define PARTICLEFOUNTAINS_H

#include "Tools.h"
#include "StaticDecorationBase.h"

//----------------------------------------------------------------------------
class ParticleFountainSurfaces : public OrientatingSurfacesBase
{
    SURFACES_SINGLETON_OBJECT(ParticleFountainSurfaces);
};



//----------------------------------------------------------------------------
#undef DECLARE_UPDATE_STATE
#define DECLARE_UPDATE_STATE(c) \
    class c; \
    friend class c; \
    class c : public UpdateState \
    { \
        STATE_OBJECT(c); \
      public: \
        c() {} \
        ~c() {} \
        void update(ParticleFountainBase *f); \
    }


//----------------------------------------------------------------------------
class ParticleFountainBase : public StaticDecorationBase,
                             public AnimatedDecorationBase,
                             public OrientatingDecorationBase
{
    //------------------------------------------------------------------------
    class UpdateState
    {
      protected:
        UpdateState() {}
      public:
        virtual ~UpdateState() {}

        virtual void update(ParticleFountainBase *f) = 0;
    };

    //------------------------------------------------------------------------
    DECLARE_UPDATE_STATE(Initial);
    DECLARE_UPDATE_STATE(InitialDelay);
    DECLARE_UPDATE_STATE(PermanentActivation);
    DECLARE_UPDATE_STATE(Activated);
    DECLARE_UPDATE_STATE(Deactivated);


  public:

    //------------------------------------------------------------------------
    class InitializationData
        : public StaticDecorationBase::InitializationData,
          public AnimatedDecorationBase::InitializationData,
          public OrientatingDecorationBase::InitializationData
    {
      public:
        InitializationData(const XMLNode *fountainNode)
            throw (XMLException);
        ~InitializationData();

        unsigned speed;
        unsigned lifeTime;
        unsigned scatter;
    };


    virtual ~ParticleFountainBase();

    static ParticleFountainBase *create(const XMLNode *fountainNode)
        throw (Exception);

    virtual int getParticleInitialXVelocity() const = 0;
    virtual int getParticleInitialYVelocity() const = 0;

    inline unsigned getParticleSpeed() const
    {
        return m_particleSpeed;
    }
    inline unsigned getParticleLifeTime() const
    {
        return m_particleLifeTime;
    }
    inline unsigned getParticleScatter() const
    {
        return m_particleScatter;
    }

    inline void setUpdateState(UpdateState *state)
    {
        m_updateState = state;
    }

  protected:
    ParticleFountainBase(const InitializationData &init);

    static ParticleFountainBase *create(const InitializationData &init);

    unsigned m_frameCounter;

  private:
    //------------------------------------------------------------------------
    void do_update();

    //------------------------------------------------------------------------
    DECLARE_VISITOR_API();

    //------------------------------------------------------------------------
    unsigned m_particleSpeed;
    unsigned m_particleLifeTime;
    unsigned m_particleScatter;

    UpdateState *m_updateState;
};

//----------------------------------------------------------------------------
class TopParticleFountain : public ParticleFountainBase
{
  public:
    TopParticleFountain(const InitializationData &init);
    ~TopParticleFountain();

    int getParticleInitialXVelocity() const;
    int getParticleInitialYVelocity() const;
};

//----------------------------------------------------------------------------
class BottomParticleFountain : public ParticleFountainBase
{
  public:
    BottomParticleFountain(const InitializationData &init);
    ~BottomParticleFountain();

    int getParticleInitialXVelocity() const;
    int getParticleInitialYVelocity() const;
};

//----------------------------------------------------------------------------
class LeftParticleFountain : public ParticleFountainBase
{
  public:
    LeftParticleFountain(const InitializationData &init);
    ~LeftParticleFountain();

    int getParticleInitialXVelocity() const;
    int getParticleInitialYVelocity() const;
};

//----------------------------------------------------------------------------
class RightParticleFountain : public ParticleFountainBase
{
  public:
    RightParticleFountain(const InitializationData &init);
    ~RightParticleFountain();

    int getParticleInitialXVelocity() const;
    int getParticleInitialYVelocity() const;
};

#endif //PARTICLEFOUNTAINS_H
