#include <cppunit/TestFixture.h>
#include <cppunit/extensions/HelperMacros.h>

#include "Configuration.h"
#include "Tiles.h"
#include "DecorationBase.h"
#include "GameControlBase.h"
#include "LevelReader.h"
#include "GameBridge.h"
#include "PlayerConfiguration.h"


//----------------------------------------------------------------------------
class LevelTest : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE(LevelTest);

    CPPUNIT_TEST(testXMLContentOfAllLevels);

    CPPUNIT_TEST_SUITE_END();

public:

    //------------------------------------------------------------------------
    void setUp()
    {
        Configuration::getInstance()->searchDataDir();

        TileCategories::init();
        SurfacesBase::init();

        PlayerConfiguration::create();

        GameInterface::setInstance(GameBridge::getInstance());
    }

    //------------------------------------------------------------------------
    void tearDown() 
    {
        PlayerConfiguration::destroy();

        SurfacesBase::destroy();
        TileCategories::destroy();
    }


protected:

    //------------------------------------------------------------------------
    void testXMLContentOfAllLevels()
    {
        MissionReader mr;
        LevelReader lr;
        for (MissionReader::const_iterator iter = mr.begin();
             iter != mr.end(); ++iter)
        {
            lr.init(iter->first.c_str());

            for (LevelReader::const_iterator iter2 = lr.begin();
                 iter2 != lr.end(); ++iter2)
            {
                GameControlBase::init(lr.getMission().c_str(), iter2->c_str());
            }
        }

        GameControlBase::destroy();
    }
};

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(LevelTest, "Level");
